clear all
global dict pars1 pars2 pars3
% Phantom 
load phantom;
x = xo;
n = length(x);  % Image width.
maxIters=50;

% Dictionary of curvelets.
dict='CURVWRAP2';pars1=4;pars2=0;pars3=0;

% Coefficient vector length.
p = SizeOfDict2(n,dict,pars1,pars2,pars3);
tightFrame = p/(n*n); % The dictionary is a tight frame with constant p/(n*n).

% Corrupt by multiplicative noise: mean of iid ~ Exp(sigma). This sum is distributed ~ Gamma(K,K/sigma).
sigma = 1;
K = 10;
noise = reshape(mean(exponential_rnd(1/sigma,n*n,K),2),n,n);
y = x .* noise;
v = log(y);

% Denoise v by hard-thresholding in the curvelet (tight) frame with a small threshold => outliers in the coefficients domain.
thd = 2;
coeffv = FastA2(v,dict,pars1,pars2,pars3);
cind   = fdct_wrapping_range(n,nextpow2(n)-pars1+1); % To get wedges indices in a vectorized form.
coeffv(prod(cind{1}.sw)+1:end) = HardThresh(coeffv(prod(cind{1}.sw)+1:end),thd*sqrt(psi(1,K))); % Do not threshold approximation band.
yden   = exp(FastS2(coeffv,n,dict,pars1,pars2,pars3)/tightFrame);


tvoptions.dimension = '2';    % 2 for 2D images. Other fields are set to default values.
tvoptions.numdeep = 8;        % Depth of the dyadic search for the fast TV prox.
tvoptions.lmin = min(v(:));	% The solution is truncated to lmin/lmax.
tvoptions.lmax = max(v(:));
gamm = 10;		      % Relaxation parameter for Douglas-Rachford iteration.
% Regularization parameter.
lambdaC= 1.5*[0.7 3];

tic;yhatTVCL1DR= SolveTVL1CoeffsMultDouglasRachford('FastDictOp2D', coeffv, n*n, p, tvoptions, lambdaC/tightFrame, gamm, tightFrame, maxIters, 0, 0);timeTVCL1DR=toc
tvoptions.mu = 0.249;tvoptions.iterstv = 400;
tic;yhatTVDNFB = perform_prox_tv2D(v,0.25,tvoptions);timeTVDNFB=toc
tic;yhatBlock = den2d_blockcurvwrap(v,2,6,4,sqrt(psi(1,K)));timeBlock=toc

% Take exp of the denoised log(image) and correct for bias due to exp of an estimate.
yhatTVCL1DR = exp(yhatTVCL1DR)*(1+psi(1,K)/2);
yhatTVDNFB  = exp(yhatTVDNFB)*(1+psi(1,K)/2);
yhatBlock   = exp(yhatBlock)*(1+psi(1,K)/2);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Display recovered images.
figure(1)
subplot(221); 
imagesc(x);axis image;rmaxis
colormap('gray')
title(sprintf('Original Phantom image'));

subplot(222); 
imagesc(y);axis image;rmaxis
title(sprintf('Noisy image multiplicative noise \\Sigma_{k=1}^{%d} X_k/K , X_k ~ Exp(%g)',K,sigma));

subplot(223); 
[h,t]=hist(noise(:)',100);dt=diff(t);plot(t,[h'/sum(h)/dt(1) gamma_pdf(t',K,K/sigma)]);
legend('Empirical','Theoretical');
title(sprintf('PDF of \\Sigma_{k=1}^{%d} X_k/K , X_k ~ Exp(%g)',K,sigma));

subplot(224); 
[h,t]=hist(log(noise(:)'),100);dt=diff(t);plot(t,[h'/sum(h)/dt(1) (K/sigma)^K/gamma(K)*exp(K*(t'-exp(t')/sigma))]);
legend('Empirical','Theoretical');
title(sprintf('PDF of log(\\Sigma_{k=1}^{%d} X_k/K) , X_k ~ Exp(%g)',K,sigma));

figure(2)
subplot(231); 
imagesc(x);axis image;rmaxis
colormap('gray')
title(sprintf('Original Phantom image'));

subplot(232); 
imagesc(y);axis image;rmaxis
colormap('gray')
title(sprintf('Noisy image multiplicative noise \\Sigma_{k=1}^{%d} X_k/K , X_k ~ Exp(%g)',K,sigma));

subplot(233); 
imagesc(clamp(yden,min(xo(:)),max(xo(:))));axis image;rmaxis
title(sprintf('First estimate with threshold %g (Exp(Den(log(noisy)))) \n PSNR=%.4g dB MAE=%.4g',thd*sqrt(psi(1,K)),psnr(x(:),yden(:)),norm(x(:)-yden(:),1)/(n*n)));

subplot(234); 
imagesc(clamp(yhatBlock,min(xo(:)),max(xo(:))));axis image;rmaxis
title(sprintf('Block-Stein on log(image) \n PSNR=%.4g dB MAE=%.4g',psnr(x(:),yhatBlock(:)),norm(x(:)-yhatBlock(:),1)/(n*n)));

subplot(235); 
imagesc(reshape(clamp(yhatTVCL1DR,min(xo(:)),max(xo(:))),n,n));axis image;rmaxis
title(sprintf('TVL1-DR on coeffs of log(first estimate) Iter=%d \n PSNR=%.4g dB MAE=%.4g',maxIters,psnr(x(:),yhatTVCL1DR(:)),norm(x(:)-yhatTVCL1DR(:),1)/(n*n)));

subplot(236); 
imagesc(reshape(clamp(yhatTVDNFB,min(xo(:)),max(xo(:))),n,n));axis image;rmaxis
title(sprintf('TVL2-FB on log(image) Iter=%d \n PSNR=%.4g dB MAE=%.4g',tvoptions.iterstv,psnr(x(:),yhatTVDNFB(:)),norm(x(:)-yhatTVDNFB(:),1)/(n*n)));

%saveas(gcf,'2D/Datasets/phantomTVL1Mult2D.fig','fig');

resTVCL1DR = xo - reshape(yhatTVCL1DR,n,n);
resBlock   = xo - yhatBlock;
resTVDNFB  = xo - reshape(yhatTVDNFB,n,n);
mn = max([min(resTVCL1DR(:)) min(resBlock(:)) min(resTVDNFB(:))]);
mx = min([max(resTVCL1DR(:)) max(resBlock(:)) max(resTVDNFB(:))]);

figure(3)
subplot(131);
colormap('gray')
imagesc(clamp(resBlock,mn,mx));axis image;rmaxis
title(sprintf('Residuals \n Block-Stein on log(image)'));

subplot(132); 
imagesc(clamp(resTVCL1DR,mn,mx));axis image;rmaxis
title(sprintf('Residuals \n TVL1-DR on coeffs of log(first estimate)'));

subplot(133);
colormap('gray')
imagesc(clamp(resTVDNFB,mn,mx));axis image;rmaxis
title(sprintf('Residuals \n TVL2-FB on log(image)'));

%saveas(gcf,'2D/Datasets/phantomresidualsTVL1Mult2D.fig','fig');

%
% Part of CurvL1MultDenoise Version:100
% Written by: Jalal Fadili, GREYC CNRS-ENSICAEN-Univ. Caen
%      	      Mila Nikolova, CMLA CNRS-ENS Cachan
%	      Sylvain Durand, MAP5 CNRS-Univ. Paris 5
% Created March 2009
% This is Copyrighted Material
% E-mail: Jalal.Fadili@greyc.ensicaen.fr
%
% Modified Wed Dec 16 18:41:37 CET 2009
%

